#version 330 core

layout(location = 0) in vec3 vertexPosition;
layout(location = 1) in vec3 vertexNormal;
layout(location = 2) in vec2 vertexUV;

layout(location = 3) in vec3 vertexTangent;
layout(location = 4) in vec3 vertexBitangent;

uniform mat4 mvpMatrix;
uniform mat4 modelMatrix;
uniform mat3 normalMatrix;
uniform mat4 DepthBiasMVP;
uniform vec3 lightDir;
uniform vec3 eye;

out vec3 worldPosition;
out vec3 worldNormal;
out vec2 UV;
out vec4 ShadowCoord;

out vec3 tangentLightDir;
out vec3 tangentEyeDir;

void main() {
	UV = vertexUV;
	worldPosition = (mvpMatrix * vec4(vertexPosition, 1)).xyz;
	worldNormal = normalMatrix * vertexNormal;
	gl_Position = mvpMatrix * vec4(vertexPosition, 1);
	ShadowCoord = DepthBiasMVP * vec4(vertexPosition,1);
	
	vec3 worldTangent = mat3(modelMatrix) * vertexTangent;
	vec3 worldBitangent = mat3(modelMatrix) * vertexBitangent;
	
	mat3 tsm = inverse(mat3(worldTangent, worldBitangent, worldNormal));
	tangentLightDir = tsm * lightDir;
	tangentEyeDir = tsm * normalize(eye-worldPosition);
}
